// Liest eine Datei des DWD mit extension .asc
// Die Datei startet mit 6 Zeilen im Format text wert, z.B. NCOLS 816
// Erzeugt daraus gnuplot x,y,z (GRID,Matrix) Daten mit Leerzeile am Ende
// einer Matrixzeile
// Jede Zeile der Ausgabedatei hat Format x y z, wobei z=gelesener Wert
// Ausgabe ueber stdout (redirection noetig)
// E.Huckert 04-2022

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

 int logging = 0;

// --------------------------------------------
void usage()
{
  fprintf(stderr, "Aufruf: prepareDWDData inpFile [-v] >outfile\n");
  fprintf(stderr, "E.Huckert 04-2022\n");
}

// ---------------------------------------------------------------------
int getNoFields(const char *buf, 
                const char   sep)   // field separator character  
{
  int nof    = 0;
  for (int n=0; buf[n] != 0; n++)
  {
    if ((buf[n] == sep) || (buf[n] == '\n')) 
    {
      if (buf[n+1] != sep)
        nof++;
    }
  }
  return nof;
}   // end getNoFields()

// -----------------------------------------------------------------------
// Parse a string and return the desired field content
// returns: the number of extracted characters
//          pos: the field absolute end pos. in buf
//          The returned content (retbuf) is zero terminated
int getField(const char  *buf, 
                   int   no_field,   // from 0 on
                   char  *retbuf,    // return buffer
                   int   lretbuf,    // max. length of retbuf
                   int   *pos,       // return position
                   const char sep1)  // field separator character  
{
  int  m, n, nof;
  int  ret = 0;
  //
  m    = 0;
  nof  = 0;
  *pos = 0;
  //
  for (n=0; buf[n] != 0; n++)
  {
    if (((buf[n] == sep1) && (buf[n+1] != sep1)) ||
        (buf[n] == '\n'))
    {
      retbuf[m] = 0;
      if (m > 0)
      {
        // consider/count only fields that are filled
        nof++;
        if (nof > no_field) 
        {
          *pos = n;
          ret = m;
          goto zurueck;
        }
      }   // end if (m > 0)
      m = 0;
    }
    else
    {
      if (m < (lretbuf - 1))
        retbuf[m++] = buf[n];
    }
  }   // for n ...
  //
  zurueck:
  return ret;
}   // end getField()

// --------------------------------------------
int main(int argc, char *argv[])
{
  FILE *ifd = NULL;
  static char buf[8192];
  int x , y , z;
  int m = 0;
  int ret = 0;
  int noCols= 0;
  int noRows = 0;
  int noFields = 0;
  int fieldPos = 0;
  char zbuf[64];
  unsigned count = 0;

  for (int n=1; n < argc; n++)
  {
    if (strcmp(argv[n], "-v") == 0)
      logging = 1;
  }
  //
  ifd = fopen(argv[1], "r");
  if (ifd == NULL)
  {
    fprintf(stderr, "ERROR input file\n");
    goto zurueck;
  }
  y = 0;
  count = 0;
  //
  while (1)
  {
    if (fgets(buf, sizeof(buf) - 1, ifd) == NULL)
      break;
    count++;
    if (count <= 6) 
    {
      if (strncmp(buf, "NCOLS ", 6) == 0)
        noCols = atoi(buf + 6);
      if (strncmp(buf, "NROWS ", 6) == 0)
        noRows = atoi(buf + 6);
      // skipt the nonnumeric lines
      continue; 
    }
    noFields = getNoFields(buf, ' ');
    if (logging)
      printf("length=%d noCols=%d noRows=%d noFields=%d\n", 
              strlen(buf), noCols, noRows, noFields);
    // here in the numeric data
    for (int n=0; n < noCols; n++)
    {
      // get the z-values
      ret = getField(buf, n, zbuf, sizeof(zbuf) - 1, 
                     &fieldPos, ' ');
      if (logging)
        printf("zbuf=%s fieldPos=%d ret=%d\n", zbuf, fieldPos, ret);
      if (ret <= 0)
        break;
      z = atoi(zbuf);
      printf("%d %d %d\n", n, y, z); // output in x,y,z format
    }   // end for n ...
    // output an empty line (=end of a row for gnuplot)
    printf("\n");
    y++;
  }   // end while (1)
  //
  zurueck:
  fflush(stdout);
  if (ifd != NULL)
    fclose(ifd);
  return (int)count;
}   // end main()
